//Code from https://nodemcu.readthedocs.io/en/latest/en/modules/tm1829/


#include "user_interface.h"


#define LEDCOUNT 50
uint8_t ledArr[LEDCOUNT * 3];
int stripPin = -1;
void updateLeds() {
  if (stripPin != -1)
    tm1829_write_to_pin(stripPin, ledArr, LEDCOUNT * 3);
}

void setColor(int ledNumber, byte r, byte g, byte b) {
  if (ledNumber ==0){ //First Byte must not be 255!
    r = constrain(r,0,254);
    g = constrain(g,0,254);
    b = constrain(b,0,254);
  }
  if (ledNumber < LEDCOUNT) {
    ledArr[ledNumber * 3] = g;
    ledArr[ledNumber * 3 + 1] = r;
    ledArr[ledNumber * 3 + 2] = b;
  }
}

void setAllBlack() {
  for (int i = 0; i < LEDCOUNT * 3; i++) {
    ledArr[i] = 0;
  }
  updateLeds();
}

static inline uint32_t _getCycleCount(void) {
  uint32_t cycles;
  __asm__ __volatile__("rsr %0,ccount":"=a" (cycles));
  return cycles;
}

void tm1829_write_to_pin(uint8_t pin, uint8_t *pixels, uint32_t length) {
  uint8_t *p, *end;

  p   =  pixels;
  end =  p + length;

  const uint32_t t0l  = (1000 * system_get_cpu_freq()) / 3333;  // 0.390us (spec=0.35 +- 0.15)
  const uint32_t t1l  = (1000 * system_get_cpu_freq()) / 1250;  // 0.800us (spec=0.70 +- 0.15)

  const uint32_t ttot = (1000 * system_get_cpu_freq()) / 800;   // 1.25us

  while (p != end) {

    register int i;

    register uint8_t pixel = *p++;

    ets_intr_lock();
    for (i = 7; i >= 0; i--) {
      register uint32_t pin_mask = 1 << pin;

      // Select low time
      register uint32_t tl = ((pixel >> i) & 1) ? t1l : t0l;
      register uint32_t t1, t2;

      register uint32_t t = _getCycleCount();

      t1 = t + tl;
      t2 = t + ttot;
      //digitalWrite(outputPin, LOW);
      GPIO_REG_WRITE(GPIO_OUT_W1TC_ADDRESS, pin_mask); // Set pin low
      while (_getCycleCount() < t1);

      //digitalWrite(outputPin, HIGH);
      GPIO_REG_WRITE(GPIO_OUT_W1TS_ADDRESS, pin_mask); // Set pin high
      while (_getCycleCount() < t2);
    }

    ets_intr_unlock();
  }
}


void initTM1829(int pin) {
  stripPin = pin;
  pinMode(stripPin, OUTPUT);
  digitalWrite(stripPin, HIGH);
  delay(1);
  setAllBlack();

}

